@@ -1,36 +1,47 @@
-use super::common::{ImmutableString, Range};
+use super::common::Range;
 use super::errors::*;
 use super::tokens::Token;
+use std::borrow::Cow;
+use std::str::Chars;
 
 /// Converts text into a stream of tokens.
-pub struct Scanner {
-    char_index: usize,
+pub struct Scanner<'a> {
     byte_index: usize,
     line_number: usize,
     token_start: usize,
     token_start_line: usize,
-    chars: Vec<char>, // todo: use an iterator instead?
-    current_token: Option<Token>,
-    text_length: usize,
+    char_iter: Chars<'a>,
+    char_buffer: Vec<char>,
+    current_token: Option<Token<'a>>,
+    file_text: &'a str,
 }
 
-impl Scanner {
+const CHAR_BUFFER_MAX_SIZE: usize = 6;
+
+impl<'a> Scanner<'a> {
     /// Creates a new scanner based on the provided text.
-    pub fn new(text: &str) -> Scanner {
+    pub fn new(text: &'a str) -> Scanner<'a> {
+        let mut char_iter = text.chars();
+        let mut char_buffer = Vec::with_capacity(CHAR_BUFFER_MAX_SIZE);
+        let current_char = char_iter.next();
+        if let Some(current_char) = current_char {
+            char_buffer.push(current_char);
+        }
+
         Scanner {
-            char_index: 0,
             byte_index: 0,
             line_number: 0,
             token_start: 0,
             token_start_line: 0,
-            chars: text.chars().collect(),
+            char_iter,
+            char_buffer,
             current_token: None,
-            text_length: text.len(),
+            file_text: text,
         }
     }
 
     /// Moves to and returns the next token.
-    pub fn scan(&mut self) -> Result<Option<Token>, ParseError> {
+    pub fn scan(&mut self) -> Result<Option<Token<'a>>, ParseError> {
         self.skip_whitespace();
         self.token_start = self.byte_index;
         self.token_start_line = self.line_number;
@@ -114,7 +125,7 @@ impl Scanner {
     }
 
     /// Gets the current token.
-    pub fn token(&self) -> Option<Token> {
+    pub fn token(&self) -> Option<Token<'a>> {
         self.current_token.as_ref().map(|x| x.to_owned())
     }
 
@@ -126,30 +137,34 @@ impl Scanner {
         self.create_error_for_start_and_line(self.byte_index, self.line_number, message)
     }
 
-    pub(super) fn create_error_for_start_and_line(&self, start: usize, start_line: usize, message: &str) -> ParseError {
+    pub(super) fn create_error_for_start_and_line(
+        &self,
+        start: usize,
+        start_line: usize,
+        message: &str,
+    ) -> ParseError {
         let range = Range {
             start,
             start_line,
-            end: std::cmp::min(self.byte_index + 1, self.text_length),
+            end: std::cmp::min(self.byte_index + 1, self.file_text.len()),
             end_line: self.line_number,
         };
         self.create_error_for_range(range, message)
     }
 
     pub(super) fn create_error_for_range(&self, range: Range, message: &str) -> ParseError {
-        use std::iter::FromIterator;
-        let file_text = String::from_iter(&self.chars);
-        ParseError::new(range, message, &file_text)
+        ParseError::new(range, message, &self.file_text)
     }
 
-    fn parse_string(&mut self) -> Result<Token, ParseError> {
+    fn parse_string(&mut self) -> Result<Token<'a>, ParseError> {
         debug_assert!(
             self.current_char() == Some('\'') || self.current_char() == Some('"'),
             "Expected \", was {:?}",
             self.current_char()
         );
         let is_double_quote = self.current_char() == Some('"');
-        let mut text = String::new();
+        let mut last_start_byte_index = self.byte_index + 1;
+        let mut text: Option<String> = None;
         let mut last_was_backslash = false;
         let mut found_end_string = false;
 
@@ -158,81 +173,80 @@ impl Scanner {
                 let escape_start = self.byte_index - 1; // -1 for backslash
                 let escape_start_line = self.line_number;
                 match current_char {
-                    '"' => {
-                        if !is_double_quote {
-                            return Err(self.create_error_for_start_and_line(
-                                escape_start,
-                                escape_start_line,
-                                "Invalid escape in single quote string",
-                            ));
-                        } else {
-                            text.push(current_char);
-                        }
-                    }
-                    '\'' => {
-                        if is_double_quote {
-                            return Err(self.create_error_for_start_and_line(
-                                escape_start,
-                                escape_start_line,
-                                "Invalid escape in double quote string",
-                            ));
-                        } else {
-                            text.push(current_char);
-                        }
-                    }
-                    '\\' => {
-                        text.push('\\');
-                    }
-                    '/' => {
-                        text.push('/');
-                    }
-                    'b' => {
-                        text.push('\u{08}');
-                    }
-                    'f' => {
-                        text.push('\u{0C}');
-                    }
-                    'n' => {
-                        text.push('\n');
-                    }
-                    'r' => {
-                        text.push('\r');
-                    }
-                    't' => {
-                        text.push('\t');
-                    }
-                    'u' => {
-                        let mut hex_text = String::new();
-                        // expect four hex values
-                        for _ in 0..4 {
-                            let current_char = self.move_next_char();
-                            if !self.is_hex() {
+                    '"' | '\'' | '\\' | '/' | 'b' | 'f' | 'u' | 'r' | 'n' | 't' => {
+                        if current_char == '"' {
+                            if !is_double_quote {
                                 return Err(self.create_error_for_start_and_line(
                                     escape_start,
                                     escape_start_line,
-                                    "Expected four hex digits",
+                                    "Invalid escape in single quote string",
                                 ));
                             }
-                            if let Some(current_char) = current_char {
-                                hex_text.push(current_char);
+                        } else if current_char == '\'' {
+                            if is_double_quote {
+                                return Err(self.create_error_for_start_and_line(
+                                    escape_start,
+                                    escape_start_line,
+                                    "Invalid escape in double quote string",
+                                ));
                             }
                         }
 
-                        let hex_u32 = u32::from_str_radix(&hex_text, 16);
-                        let hex_char = match hex_u32.ok().map(|hex_u32| std::char::from_u32(hex_u32)).flatten() {
-                            Some(hex_char) => hex_char,
-                            None => {
-                                return Err(self.create_error_for_start_and_line(
-                                    escape_start,
-                                    escape_start_line,
-                                    &format!(
-                                        "Invalid unicode escape sequence. '{}' is not a valid UTF8 character",
-                                        hex_text
-                                    ),
-                                ));
+                        let previous_text = &self.file_text[last_start_byte_index..escape_start];
+                        if text.is_none() {
+                            text = Some(String::new());
+                        }
+                        let text = text.as_mut().unwrap();
+                        text.push_str(previous_text);
+                        if current_char == 'u' {
+                            let mut hex_text = String::new();
+                            // expect four hex values
+                            for _ in 0..4 {
+                                let current_char = self.move_next_char();
+                                if !self.is_hex() {
+                                    return Err(self.create_error_for_start_and_line(
+                                        escape_start,
+                                        escape_start_line,
+                                        "Expected four hex digits",
+                                    ));
+                                }
+                                if let Some(current_char) = current_char {
+                                    hex_text.push(current_char);
+                                }
                             }
-                        };
-                        text.push(hex_char);
+
+                            let hex_u32 = u32::from_str_radix(&hex_text, 16);
+                            let hex_char = match hex_u32
+                                .ok()
+                                .map(|hex_u32| std::char::from_u32(hex_u32))
+                                .flatten()
+                            {
+                                Some(hex_char) => hex_char,
+                                None => {
+                                    return Err(self.create_error_for_start_and_line(
+                                        escape_start,
+                                        escape_start_line,
+                                        &format!(
+                                            "Invalid unicode escape sequence. '{}' is not a valid UTF8 character",
+                                            hex_text
+                                        ),
+                                    ));
+                                }
+                            };
+                            text.push(hex_char);
+                            last_start_byte_index = self.byte_index
+                                + self.current_char().map(|c| c.len_utf8()).unwrap_or(0);
+                        } else {
+                            text.push(match current_char {
+                                'b' => '\u{08}',
+                                'f' => '\u{0C}',
+                                't' => '\t',
+                                'r' => '\r',
+                                'n' => '\n',
+                                _ => current_char,
+                            });
+                            last_start_byte_index = self.byte_index + current_char.len_utf8();
+                        }
                     }
                     _ => {
                         return Err(self.create_error_for_start_and_line(
@@ -243,49 +257,52 @@ impl Scanner {
                     }
                 }
                 last_was_backslash = false;
-            } else if is_double_quote && current_char == '"' || !is_double_quote && current_char == '\'' {
+            } else if is_double_quote && current_char == '"'
+                || !is_double_quote && current_char == '\''
+            {
                 found_end_string = true;
                 break;
             } else {
                 last_was_backslash = current_char == '\\';
-                if !last_was_backslash {
-                    text.push(current_char);
-                }
             }
         }
 
         if found_end_string {
             self.move_next_char();
-            Ok(Token::String(ImmutableString::new(text)))
+            let final_segment = &self.file_text[last_start_byte_index..self.byte_index - 1];
+            Ok(Token::String(match text {
+                Some(mut text) => {
+                    text.push_str(final_segment);
+                    Cow::Owned(text)
+                }
+                None => Cow::Borrowed(final_segment),
+            }))
         } else {
             Err(self.create_error_for_current_token("Unterminated string literal"))
         }
     }
 
-    fn parse_number(&mut self) -> Result<Token, ParseError> {
-        let mut text = String::new();
+    fn parse_number(&mut self) -> Result<Token<'a>, ParseError> {
+        let start_byte_index = self.byte_index;
 
         if self.is_negative_sign() {
-            text.push('-');
             self.move_next_char();
         }
 
         if self.is_zero() {
-            text.push('0');
             self.move_next_char();
         } else if self.is_one_nine() {
-            text.push(self.current_char().unwrap());
             self.move_next_char();
             while self.is_digit() {
-                text.push(self.current_char().unwrap());
                 self.move_next_char();
             }
         } else {
-            return Err(self.create_error_for_current_char("Expected a digit to follow a negative sign"));
+            return Err(
+                self.create_error_for_current_char("Expected a digit to follow a negative sign")
+            );
         }
 
         if self.is_decimal_point() {
-            text.push('.');
             self.move_next_char();
 
             if !self.is_digit() {
@@ -293,74 +310,72 @@ impl Scanner {
             }
 
             while self.is_digit() {
-                text.push(self.current_char().unwrap());
                 self.move_next_char();
             }
         }
 
         match self.current_char() {
-            Some('e') | Some('E') => {
-                text.push(self.current_char().unwrap());
-                match self.move_next_char() {
-                    Some('-') | Some('+') => {
-                        text.push(self.current_char().unwrap());
-                        self.move_next_char();
-                        if !self.is_digit() {
-                            return Err(self.create_error_for_current_char("Expected a digit"));
-                        }
-                        while self.is_digit() {
-                            text.push(self.current_char().unwrap());
-                            self.move_next_char();
-                        }
+            Some('e') | Some('E') => match self.move_next_char() {
+                Some('-') | Some('+') => {
+                    self.move_next_char();
+                    if !self.is_digit() {
+                        return Err(self.create_error_for_current_char("Expected a digit"));
                     }
-                    _ => {
-                        return Err(
-                            self.create_error_for_current_char("Expected plus or minus symbol in number literal")
-                        );
+                    while self.is_digit() {
+                        self.move_next_char();
                     }
                 }
-            }
+                _ => {
+                    return Err(self.create_error_for_current_char(
+                        "Expected plus or minus symbol in number literal",
+                    ));
+                }
+            },
             _ => {}
         }
 
-        Ok(Token::Number(ImmutableString::new(text)))
+        let end_byte_index = self.byte_index;
+        Ok(Token::Number(
+            &self.file_text[start_byte_index..end_byte_index],
+        ))
     }
 
-    fn parse_comment_line(&mut self) -> Token {
-        let mut text = String::new();
+    fn parse_comment_line(&mut self) -> Token<'a> {
         self.assert_then_move_char('/');
         #[cfg(debug_assertions)]
         self.assert_char('/');
 
-        while let Some(current_char) = self.move_next_char() {
+        let start_byte_index = self.byte_index + 1;
+        while let Some(_) = self.move_next_char() {
             if self.is_new_line() {
                 break;
             }
-            text.push(current_char);
         }
 
-        Token::CommentLine(ImmutableString::new(text))
+        Token::CommentLine(&self.file_text[start_byte_index..self.byte_index])
     }
 
-    fn parse_comment_block(&mut self) -> Result<Token, ParseError> {
-        let mut text = String::new();
+    fn parse_comment_block(&mut self) -> Result<Token<'a>, ParseError> {
         self.assert_then_move_char('/');
         #[cfg(debug_assertions)]
         self.assert_char('*');
         let mut found_end = false;
 
+        let start_byte_index = self.byte_index + 1;
         while let Some(current_char) = self.move_next_char() {
             if current_char == '*' && self.peek_char() == Some('/') {
                 found_end = true;
                 break;
             }
-            text.push(current_char);
         }
 
         if found_end {
+            let end_byte_index = self.byte_index;
             self.assert_then_move_char('*');
             self.assert_then_move_char('/');
-            Ok(Token::CommentBlock(ImmutableString::new(text)))
+            Ok(Token::CommentBlock(
+                &self.file_text[start_byte_index..end_byte_index],
+            ))
         } else {
             Err(self.create_error_for_current_token("Unterminated comment block"))
         }
@@ -377,54 +392,59 @@ impl Scanner {
     }
 
     fn try_move_word(&mut self, text: &str) -> bool {
-        // todo: debug assert no newlines
-        let mut char_index = self.char_index;
-        let mut byte_index = self.byte_index;
+        let mut char_index = 0;
         for c in text.chars() {
-            if let Some(current_char) = self.chars.get(char_index) {
-                if *current_char != c {
+            if let Some(current_char) = self.peek_char_offset(char_index) {
+                if current_char != c {
                     return false;
                 }
 
                 char_index += 1;
-                byte_index += current_char.len_utf8();
             } else {
                 return false;
             }
         }
 
-        if let Some(next_char) = self.chars.get(char_index) {
+        if let Some(next_char) = self.peek_char_offset(char_index) {
             if next_char.is_alphanumeric() {
                 return false;
             }
         }
 
-        self.char_index = char_index;
-        self.byte_index = byte_index;
+        for _ in 0..char_index {
+            self.move_next_char();
+        }
+
         true
     }
 
-    fn parse_word(&mut self) -> Result<Token, ParseError> {
-        let mut text = String::new();
+    fn parse_word(&mut self) -> Result<Token<'a>, ParseError> {
+        let start_byte_index = self.byte_index;
 
         while let Some(current_char) = self.current_char() {
-            if current_char.is_whitespace() || current_char == '\r' || current_char == '\n' || current_char == ':' {
+            if current_char.is_whitespace()
+                || current_char == '\r'
+                || current_char == '\n'
+                || current_char == ':'
+            {
                 break;
             }
             if !current_char.is_alphanumeric() && current_char != '-' {
                 return Err(self.create_error_for_current_token("Unexpected token"));
             }
 
-            text.push(current_char);
-
             self.move_next_char();
         }
 
-        if text.len() == 0 {
+        let end_byte_index = self.byte_index;
+
+        if end_byte_index - start_byte_index == 0 {
             return Err(self.create_error_for_current_token("Unexpected token"));
         }
 
-        Ok(Token::Word(ImmutableString::new(text)))
+        Ok(Token::Word(
+            &self.file_text[start_byte_index..end_byte_index],
+        ))
     }
 
     fn assert_then_move_char(&mut self, _character: char) {
@@ -446,25 +466,55 @@ impl Scanner {
     }
 
     fn move_next_char(&mut self) -> Option<char> {
-        if let Some(current_char) = self.current_char() {
+        if let Some(&current_char) = self.char_buffer.get(0) {
             if current_char == '\n' {
                 self.line_number += 1;
             }
-            self.char_index += 1;
+
+            // shift the entire array to the left then pop the last item
+            for i in 1..self.char_buffer.len() {
+                self.char_buffer[i - 1] = self.char_buffer[i];
+            }
+            self.char_buffer.pop();
+
+            if self.char_buffer.is_empty() {
+                if let Some(new_char) = self.char_iter.next() {
+                    self.char_buffer.push(new_char);
+                }
+            }
+
             self.byte_index += current_char.len_utf8();
         }
+
         self.current_char()
     }
 
-    fn peek_char(&self) -> Option<char> {
-        self.chars.get(self.char_index + 1).map(|x| x.to_owned())
+    fn peek_char(&mut self) -> Option<char> {
+        self.peek_char_offset(1)
+    }
+
+    fn peek_char_offset(&mut self, offset: usize) -> Option<char> {
+        // fill the char buffer
+        for _ in self.char_buffer.len()..offset + 1 {
+            if let Some(next_char) = self.char_iter.next() {
+                self.char_buffer.push(next_char);
+            } else {
+                // end of string
+                return None;
+            }
+        }
+
+        // should not exceed this
+        debug_assert!(self.char_buffer.len() <= CHAR_BUFFER_MAX_SIZE);
+
+        self.char_buffer.get(offset).map(|c| *c)
     }
 
     fn current_char(&self) -> Option<char> {
-        self.chars.get(self.char_index).map(|x| x.to_owned())
+        self.char_buffer.get(0).map(|c| *c)
     }
 
-    fn is_new_line(&self) -> bool {
+    fn is_new_line(&mut self) -> bool {
         match self.current_char() {
             Some('\n') => true,
             Some('\r') => self.peek_char() == Some('\n'),
@@ -476,7 +526,8 @@ impl Scanner {
         self.is_digit()
             || match self.current_char() {
                 Some(current_char) => {
-                    current_char >= 'a' && current_char <= 'f' || current_char >= 'A' && current_char <= 'F'
+                    current_char >= 'a' && current_char <= 'f'
+                        || current_char >= 'A' && current_char <= 'F'
                 }
                 _ => false,
             }
@@ -508,18 +559,17 @@ impl Scanner {
 
 #[cfg(test)]
 mod tests {
-    use super::super::common::ImmutableString;
     use super::super::tokens::Token;
     use super::*;
 
     #[test]
     fn it_tokenizes_string() {
         assert_has_tokens(
-            r#""t\"est", "\t\r\n\n\u0020","#,
+            r#""t\"est", "\t\r\n\n\u0020 test\n other","#,
             vec![
-                Token::String(ImmutableString::from(r#"t"est"#)),
+                Token::String(Cow::Borrowed(r#"t"est"#)),
                 Token::Comma,
-                Token::String(ImmutableString::from("\t\r\n\n ")),
+                Token::String(Cow::Borrowed("\t\r\n\n  test\n other")),
                 Token::Comma,
             ],
         );
@@ -538,9 +588,9 @@ mod tests {
         assert_has_tokens(
             r#"'t\'est','a',"#,
             vec![
-                Token::String(ImmutableString::from(r#"t'est"#)),
+                Token::String(Cow::Borrowed(r#"t'est"#)),
                 Token::Comma,
-                Token::String(ImmutableString::from("a")),
+                Token::String(Cow::Borrowed("a")),
                 Token::Comma,
             ],
         );
@@ -564,15 +614,15 @@ mod tests {
         assert_has_tokens(
             "0, 0.123, -198, 0e-345, 0.3e+025,",
             vec![
-                Token::Number(ImmutableString::from("0")),
+                Token::Number("0"),
                 Token::Comma,
-                Token::Number(ImmutableString::from("0.123")),
+                Token::Number("0.123"),
                 Token::Comma,
-                Token::Number(ImmutableString::from("-198")),
+                Token::Number("-198"),
                 Token::Comma,
-                Token::Number(ImmutableString::from("0e-345")),
+                Token::Number("0e-345"),
                 Token::Comma,
-                Token::Number(ImmutableString::from("0.3e+025")),
+                Token::Number("0.3e+025"),
                 Token::Comma,
             ],
         );
@@ -604,9 +654,9 @@ mod tests {
         assert_has_tokens(
             "//test\n//t\r\n// test\n,",
             vec![
-                Token::CommentLine(ImmutableString::from("test")),
-                Token::CommentLine(ImmutableString::from("t")),
-                Token::CommentLine(ImmutableString::from(" test")),
+                Token::CommentLine("test"),
+                Token::CommentLine("t"),
+                Token::CommentLine(" test"),
                 Token::Comma,
             ],
         );
@@ -617,8 +667,8 @@ mod tests {
         assert_has_tokens(
             "/*test\n *//* test*/,",
             vec![
-                Token::CommentBlock(ImmutableString::from("test\n ")),
-                Token::CommentBlock(ImmutableString::from(" test")),
+                Token::CommentBlock("test\n "),
+                Token::CommentBlock(" test"),
                 Token::Comma,
             ],
         );
