@@ -1,30 +1,31 @@
-use super::common::{ImmutableString, Range, Ranged};
+use super::common::{Range, Ranged};
+use std::borrow::Cow;
 
 /// A token found while scanning.
 #[derive(Debug, PartialEq, Clone)]
-pub enum Token {
+pub enum Token<'a> {
     OpenBrace,
     CloseBrace,
     OpenBracket,
     CloseBracket,
     Comma,
     Colon,
-    String(ImmutableString),
-    Word(ImmutableString),
+    String(Cow<'a, str>),
+    Word(&'a str),
     Boolean(bool),
-    Number(ImmutableString),
+    Number(&'a str),
     Null,
-    CommentLine(ImmutableString),
-    CommentBlock(ImmutableString),
+    CommentLine(&'a str),
+    CommentBlock(&'a str),
 }
 
 /// A token with positional information.
-pub struct TokenAndRange {
+pub struct TokenAndRange<'a> {
     pub range: Range,
-    pub token: Token,
+    pub token: Token<'a>,
 }
 
-impl Ranged for TokenAndRange {
+impl<'a> Ranged for TokenAndRange<'a> {
     fn range(&self) -> &Range {
         &self.range
     }
