@@ -109,15 +109,15 @@ fn range_to_test_str(range: &Range) -> String {
 }
 
 fn string_lit_to_test_str(lit: &StringLit) -> String {
-    lit_to_test_str("string", lit.value.as_str(), &lit.range)
+    lit_to_test_str("string", &lit.value, &lit.range)
 }
 
 fn word_lit_to_test_str(lit: &WordLit) -> String {
-    lit_to_test_str("word", lit.value.as_str(), &lit.range)
+    lit_to_test_str("word", lit.value, &lit.range)
 }
 
 fn number_lit_to_test_str(lit: &NumberLit) -> String {
-    lit_to_test_str("number", lit.value.as_str(), &lit.range)
+    lit_to_test_str("number", lit.value, &lit.range)
 }
 
 fn boolean_lit_to_test_str(lit: &BooleanLit) -> String {
@@ -128,7 +128,10 @@ fn lit_to_test_str(lit_type: &str, value: &str, range: &Range) -> String {
     let mut text = String::new();
     text.push_str("{\n");
     text.push_str(&format!("  \"type\": \"{}\",\n", lit_type));
-    text.push_str(&format!("  {},\n", range_to_test_str(range).replace("\n", "\n  ")));
+    text.push_str(&format!(
+        "  {},\n",
+        range_to_test_str(range).replace("\n", "\n  ")
+    ));
     text.push_str(&format!("  \"value\": \"{}\"\n", escape_json_str(value)));
     text.push_str("}");
     text
@@ -138,7 +141,10 @@ fn object_to_test_str(obj: &Object) -> String {
     let mut text = String::new();
     text.push_str("{\n");
     text.push_str("  \"type\": \"object\",\n");
-    text.push_str(&format!("  {},\n", range_to_test_str(&obj.range).replace("\n", "\n  ")));
+    text.push_str(&format!(
+        "  {},\n",
+        range_to_test_str(&obj.range).replace("\n", "\n  ")
+    ));
     text.push_str("  \"properties\": [");
     let prop_count = obj.properties.len();
     for (i, prop) in obj.properties.iter().enumerate() {
@@ -184,7 +190,10 @@ fn array_to_test_str(arr: &Array) -> String {
     let mut text = String::new();
     text.push_str("{\n");
     text.push_str("  \"type\": \"array\",\n");
-    text.push_str(&format!("  {},\n", range_to_test_str(&arr.range).replace("\n", "\n  ")));
+    text.push_str(&format!(
+        "  {},\n",
+        range_to_test_str(&arr.range).replace("\n", "\n  ")
+    ));
     text.push_str("  \"elements\": [");
     let elements_count = arr.elements.len();
     for (i, element) in arr.elements.iter().enumerate() {
@@ -237,11 +246,11 @@ fn comment_to_test_str(comment: &Comment) -> String {
 }
 
 fn comment_line_to_test_str(line: &CommentLine) -> String {
-    lit_to_test_str("line", &line.text.as_str(), &line.range)
+    lit_to_test_str("line", line.text, &line.range)
 }
 
 fn comment_block_to_test_str(block: &CommentBlock) -> String {
-    lit_to_test_str("block", &block.text.as_str(), &block.range)
+    lit_to_test_str("block", block.text, &block.range)
 }
 
 fn escape_json_str(text: &str) -> String {
